@@ -3,17 +3,37 @@ import json
 import numpy as np
 import pandas as pd
 from pandas import DataFrame, Series
-from shapely.geometry import mapping, shape, Point
-from shapely.geometry.base import BaseGeometry
-from six import string_types, PY3
+from six import PY3, string_types
 
-from geopandas.base import GeoPandasBase, _CoordinateIndexer
+from shapely.geometry import mapping, shape
+
+from geopandas.array import GeometryArray, from_shapely
+from geopandas.base import GeoPandasBase, is_geometry_type
 from geopandas.geoseries import GeoSeries
-from geopandas.plotting import plot_dataframe
 import geopandas.io
+from geopandas.plotting import plot_dataframe
+
+DEFAULT_GEO_COLUMN_NAME = "geometry"
 
 
-DEFAULT_GEO_COLUMN_NAME = 'geometry'
+def _ensure_geometry(data):
+    """
+    Ensure the data is of geometry dtype or converted to it.
+
+    If input is a (Geo)Series, output is a GeoSeries, otherwise output
+    is GeometryArray.
+    """
+    if is_geometry_type(data):
+        if isinstance(data, Series):
+            return GeoSeries(data)
+        return data
+    else:
+        if isinstance(data, Series):
+            out = from_shapely(np.asarray(data))
+            return GeoSeries(out, index=data.index, name=data.name)
+        else:
+            out = from_shapely(data)
+            return out
 
 
 class GeoDataFrame(GeoPandasBase, DataFrame):
@@ -31,54 +51,61 @@ class GeoDataFrame(GeoPandasBase, DataFrame):
         column on GeoDataFrame.
     """
 
-    # XXX: This will no longer be necessary in pandas 0.17
-    _internal_names = ['_data', '_cacher', '_item_cache', '_cache',
-                       'is_copy', '_subtyp', '_index',
-                       '_default_kind', '_default_fill_value', '_metadata',
-                       '__array_struct__', '__array_interface__']
-
-    _metadata = ['crs', '_geometry_column_name']
+    _metadata = ["crs", "_geometry_column_name"]
 
     _geometry_column_name = DEFAULT_GEO_COLUMN_NAME
 
     def __init__(self, *args, **kwargs):
-        crs = kwargs.pop('crs', None)
-        geometry = kwargs.pop('geometry', None)
+        crs = kwargs.pop("crs", None)
+        geometry = kwargs.pop("geometry", None)
         super(GeoDataFrame, self).__init__(*args, **kwargs)
+
+        # need to set this before calling self['geometry'], because
+        # getitem accesses crs
         self.crs = crs
+
+        # set_geometry ensures the geometry data have the proper dtype,
+        # but is not called if `geometry=None` ('geometry' column present
+        # in the data), so therefore need to ensure it here manually
+        # but within a try/except because currently non-geometries are
+        # allowed in that case
+        # TODO do we want to raise / return normal DataFrame in this case?
+        if geometry is None and "geometry" in self.columns:
+            # only if we have actual geometry values -> call set_geometry
+            try:
+                self["geometry"] = _ensure_geometry(self["geometry"].values)
+            except TypeError:
+                pass
+            else:
+                geometry = "geometry"
+
         if geometry is not None:
             self.set_geometry(geometry, inplace=True)
         self._invalidate_sindex()
 
-    # Serialize metadata (will no longer be necessary in pandas 0.17+)
-    # See https://github.com/pydata/pandas/pull/10557
-    def __getstate__(self):
-        meta = dict((k, getattr(self, k, None)) for k in self._metadata)
-        return dict(_data=self._data, _typ=self._typ,
-                    _metadata=self._metadata, **meta)
-
     def __setattr__(self, attr, val):
         # have to special case geometry b/c pandas tries to use as column...
-        if attr == 'geometry':
+        if attr == "geometry":
             object.__setattr__(self, attr, val)
         else:
             super(GeoDataFrame, self).__setattr__(attr, val)
 
     def _get_geometry(self):
         if self._geometry_column_name not in self:
-            raise AttributeError("No geometry data set yet (expected in"
-                                 " column '%s'." % self._geometry_column_name)
+            raise AttributeError(
+                "No geometry data set yet (expected in"
+                " column '%s'." % self._geometry_column_name
+            )
         return self[self._geometry_column_name]
 
     def _set_geometry(self, col):
-        # TODO: Use pandas' core.common.is_list_like() here.
-        if not isinstance(col, (list, np.ndarray, Series)):
-            raise ValueError("Must use a list-like to set the geometry"
-                             " property")
+        if not pd.api.types.is_list_like(col):
+            raise ValueError("Must use a list-like to set the geometry property")
         self.set_geometry(col, inplace=True)
 
-    geometry = property(fget=_get_geometry, fset=_set_geometry,
-                        doc="Geometry data for GeoDataFrame")
+    geometry = property(
+        fget=_get_geometry, fset=_set_geometry, doc="Geometry data for GeoDataFrame"
+    )
 
     def set_geometry(self, col, drop=False, inplace=False, crs=None):
         """
@@ -115,20 +142,20 @@ class GeoDataFrame(GeoPandasBase, DataFrame):
             frame = self.copy()
 
         if not crs:
-            crs = getattr(col, 'crs', self.crs)
+            crs = getattr(col, "crs", self.crs)
 
         to_remove = None
         geo_column_name = self._geometry_column_name
-        if isinstance(col, (Series, list, np.ndarray)):
+        if isinstance(col, (Series, list, np.ndarray, GeometryArray)):
             level = col
-        elif hasattr(col, 'ndim') and col.ndim != 1:
+        elif hasattr(col, "ndim") and col.ndim != 1:
             raise ValueError("Must pass array with one dimension only.")
         else:
             try:
                 level = frame[col].values
             except KeyError:
                 raise ValueError("Unknown column %s" % col)
-            except:
+            except Exception:
                 raise
             if drop:
                 to_remove = col
@@ -145,8 +172,7 @@ class GeoDataFrame(GeoPandasBase, DataFrame):
             level.crs = crs
 
         # Check that we are using a listlike of geometries
-        if not all(isinstance(item, BaseGeometry) or pd.isnull(item) for item in level):
-            raise TypeError("Input geometry column must contain valid geometry objects.")
+        level = _ensure_geometry(level)
         frame[geo_column_name] = level
         frame._geometry_column_name = geo_column_name
         frame.crs = crs
@@ -154,6 +180,38 @@ class GeoDataFrame(GeoPandasBase, DataFrame):
         if not inplace:
             return frame
 
+    def rename_geometry(self, col, inplace=False):
+        """
+        Renames the GeoDataFrame geometry column to
+        the specified name. By default yields a new object.
+
+        The original geometry column is replaced with the input.
+
+        Parameters
+        ----------
+        col : new geometry column label
+        inplace : boolean, default False
+            Modify the GeoDataFrame in place (do not create a new object)
+
+        Examples
+        --------
+        >>> df1 = df.rename_geometry('geom1')
+        >>> df1.geometry.name
+        'geom1'
+        >>> df.rename_geometry('geom1', inplace=True)
+        >>> df.geometry.name
+        'geom1'
+
+        Returns
+        -------
+        geodataframe : GeoDataFrame
+        """
+        geometry_col = self.geometry.name
+        if not inplace:
+            return self.rename(columns={geometry_col: col}).set_geometry(col, inplace)
+        self.rename(columns={geometry_col: col}, inplace=inplace)
+        self.set_geometry(col, inplace=inplace)
+
     @classmethod
     def from_file(cls, filename, **kwargs):
         """Alternate constructor to create a ``GeoDataFrame`` from a file.
@@ -163,7 +221,6 @@ class GeoDataFrame(GeoPandasBase, DataFrame):
 
         Parameters
         ----------
-
         filename : str
             File path or file handle to read from. Depending on which kwargs
             are included, the content of filename may vary. See
@@ -175,7 +232,6 @@ class GeoDataFrame(GeoPandasBase, DataFrame):
 
         Examples
         --------
-
         >>> df = geopandas.GeoDataFrame.from_file('nybb.shp')
         """
         return geopandas.io.file.read_file(filename, **kwargs)
@@ -218,8 +274,8 @@ class GeoDataFrame(GeoPandasBase, DataFrame):
         else:
             fs = features
 
-        if isinstance(fs, dict) and fs.get('type') == 'FeatureCollection':
-            features_lst = fs['features']
+        if isinstance(fs, dict) and fs.get("type") == "FeatureCollection":
+            features_lst = fs["features"]
         else:
             features_lst = features
 
@@ -230,17 +286,25 @@ class GeoDataFrame(GeoPandasBase, DataFrame):
             else:
                 f = f
 
-            d = {'geometry': shape(f['geometry']) if f['geometry'] else None}
-            d.update(f['properties'])
+            d = {"geometry": shape(f["geometry"]) if f["geometry"] else None}
+            d.update(f["properties"])
             rows.append(d)
         df = GeoDataFrame(rows, columns=columns)
         df.crs = crs
         return df
 
     @classmethod
-    def from_postgis(cls, sql, con, geom_col='geom', crs=None,
-                     index_col=None, coerce_float=True,
-                     parse_dates=None, params=None):
+    def from_postgis(
+        cls,
+        sql,
+        con,
+        geom_col="geom",
+        crs=None,
+        index_col=None,
+        coerce_float=True,
+        parse_dates=None,
+        params=None,
+    ):
         """
         Alternate constructor to create a ``GeoDataFrame`` from a sql query
         containing a geometry column in WKB representation.
@@ -279,13 +343,19 @@ class GeoDataFrame(GeoPandasBase, DataFrame):
         """
 
         df = geopandas.io.sql.read_postgis(
-                sql, con, geom_col=geom_col, crs=crs,
-                index_col=index_col, coerce_float=coerce_float,
-                parse_dates=parse_dates, params=params)
+            sql,
+            con,
+            geom_col=geom_col,
+            crs=crs,
+            index_col=index_col,
+            coerce_float=coerce_float,
+            parse_dates=parse_dates,
+            params=params,
+        )
 
         return df
 
-    def to_json(self, na='null', show_bbox=False, **kwargs):
+    def to_json(self, na="null", show_bbox=False, **kwargs):
         """
         Returns a GeoJSON representation of the ``GeoDataFrame`` as a string.
 
@@ -321,9 +391,9 @@ class GeoDataFrame(GeoPandasBase, DataFrame):
         This differs from `_to_geo()` only in that it is a property with
         default args instead of a method
         """
-        return self._to_geo(na='null', show_bbox=True)
+        return self._to_geo(na="null", show_bbox=True)
 
-    def iterfeatures(self, na='null', show_bbox=False):
+    def iterfeatures(self, na="null", show_bbox=False):
         """
         Returns an iterator that yields feature dictionaries that comply with
         __geo_interface__
@@ -340,8 +410,8 @@ class GeoDataFrame(GeoPandasBase, DataFrame):
 
         show_bbox : include bbox (bounds) in the geojson. default False
         """
-        if na not in ['null', 'drop', 'keep']:
-            raise ValueError('Unknown na method {0}'.format(na))
+        if na not in ["null", "drop", "keep"]:
+            raise ValueError("Unknown na method {0}".format(na))
 
         ids = np.array(self.index, copy=False)
         geometries = np.array(self[self._geometry_column_name], copy=False)
@@ -351,37 +421,40 @@ class GeoDataFrame(GeoPandasBase, DataFrame):
         if len(properties_cols) > 0:
             # convert to object to get python scalars.
             properties = self[properties_cols].astype(object).values
-            if na == 'null':
+            if na == "null":
                 properties[pd.isnull(self[properties_cols]).values] = None
 
             for i, row in enumerate(properties):
                 geom = geometries[i]
 
-                if na == 'drop':
-                    properties_items = dict((k, v) for k, v
-                                            in zip(properties_cols, row)
-                                            if not pd.isnull(v))
+                if na == "drop":
+                    properties_items = {
+                        k: v for k, v in zip(properties_cols, row) if not pd.isnull(v)
+                    }
                 else:
-                    properties_items = dict((k, v) for k, v
-                                            in zip(properties_cols, row))
+                    properties_items = {k: v for k, v in zip(properties_cols, row)}
 
-                feature = {'id': str(ids[i]),
-                           'type': 'Feature',
-                           'properties': properties_items,
-                           'geometry': mapping(geom) if geom else None}
+                feature = {
+                    "id": str(ids[i]),
+                    "type": "Feature",
+                    "properties": properties_items,
+                    "geometry": mapping(geom) if geom else None,
+                }
 
                 if show_bbox:
-                    feature['bbox'] = geom.bounds if geom else None
+                    feature["bbox"] = geom.bounds if geom else None
                 yield feature
 
         else:
             for fid, geom in zip(ids, geometries):
-                feature = {'id': str(fid),
-                           'type': 'Feature',
-                           'properties': {},
-                           'geometry': mapping(geom) if geom else None}
+                feature = {
+                    "id": str(fid),
+                    "type": "Feature",
+                    "properties": {},
+                    "geometry": mapping(geom) if geom else None,
+                }
                 if show_bbox:
-                        feature['bbox'] = geom.bounds if geom else None
+                    feature["bbox"] = geom.bounds if geom else None
                 yield feature
 
     def _to_geo(self, **kwargs):
@@ -390,16 +463,17 @@ class GeoDataFrame(GeoPandasBase, DataFrame):
         representation of the GeoDataFrame.
 
         """
-        geo = {'type': 'FeatureCollection',
-               'features': list(self.iterfeatures(**kwargs))}
+        geo = {
+            "type": "FeatureCollection",
+            "features": list(self.iterfeatures(**kwargs)),
+        }
 
-        if kwargs.get('show_bbox', False):
-            geo['bbox'] = tuple(self.total_bounds)
+        if kwargs.get("show_bbox", False):
+            geo["bbox"] = tuple(self.total_bounds)
 
         return geo
 
-    def to_file(self, filename, driver="ESRI Shapefile", schema=None,
-                **kwargs):
+    def to_file(self, filename, driver="ESRI Shapefile", schema=None, **kwargs):
         """Write the ``GeoDataFrame`` to a file.
 
         By default, an ESRI shapefile is written, but any OGR data source
@@ -426,6 +500,7 @@ class GeoDataFrame(GeoPandasBase, DataFrame):
         (zip files), etc.
         """
         from geopandas.io.file import to_file
+
         to_file(self, filename, driver, schema, **kwargs)
 
     def to_crs(self, crs=None, epsg=None, inplace=False):
@@ -506,11 +581,11 @@ class GeoDataFrame(GeoPandasBase, DataFrame):
     def __finalize__(self, other, method=None, **kwargs):
         """propagate metadata from other to self """
         # merge operation: using metadata of the left object
-        if method == 'merge':
+        if method == "merge":
             for name in self._metadata:
                 object.__setattr__(self, name, getattr(other.left, name, None))
         # concat operation: using metadata of the first object
-        elif method == 'concat':
+        elif method == "concat":
             for name in self._metadata:
                 object.__setattr__(self, name, getattr(other.objs[0], name, None))
         else:
@@ -518,25 +593,6 @@ class GeoDataFrame(GeoPandasBase, DataFrame):
                 object.__setattr__(self, name, getattr(other, name, None))
         return self
 
-    def copy(self, deep=True):
-        """
-        Make a copy of this GeoDataFrame object
-
-        Parameters
-        ----------
-        deep : boolean, default True
-            Make a deep copy, i.e. also copy data
-
-        Returns
-        -------
-        copy : GeoDataFrame
-        """
-        # FIXME: this will likely be unnecessary in pandas >= 0.13
-        data = self._data
-        if deep:
-            data = data.copy()
-        return GeoDataFrame(data).__finalize__(self)
-
     def plot(self, *args, **kwargs):
         """Generate a plot of the geometries in the ``GeoDataFrame``.
 
@@ -551,8 +607,7 @@ class GeoDataFrame(GeoPandasBase, DataFrame):
 
     plot.__doc__ = plot_dataframe.__doc__
 
-
-    def dissolve(self, by=None, aggfunc='first', as_index=True):
+    def dissolve(self, by=None, aggfunc="first", as_index=True):
         """
         Dissolve geometries within `groupby` into single observation.
         This is accomplished by applying the `unary_union` method
@@ -580,13 +635,14 @@ class GeoDataFrame(GeoPandasBase, DataFrame):
         data = self.drop(labels=self.geometry.name, axis=1)
         aggregated_data = data.groupby(by=by).agg(aggfunc)
 
-
         # Process spatial component
         def merge_geometries(block):
             merged_geom = block.unary_union
             return merged_geom
 
-        g = self.groupby(by=by, group_keys=False)[self.geometry.name].agg(merge_geometries)
+        g = self.groupby(by=by, group_keys=False)[self.geometry.name].agg(
+            merge_geometries
+        )
 
         # Aggregate
         aggregated_geometry = GeoDataFrame(g, geometry=self.geometry.name, crs=self.crs)
@@ -626,8 +682,8 @@ class GeoDataFrame(GeoPandasBase, DataFrame):
         exploded_index = exploded_geom.columns[0]
 
         df = pd.concat(
-            [df_copy.drop(df_copy._geometry_column_name, axis=1),
-             exploded_geom], axis=1)
+            [df_copy.drop(df_copy._geometry_column_name, axis=1), exploded_geom], axis=1
+        )
         # reset to MultiIndex, otherwise df index is only first level of
         # exploded GeoSeries index.
         df.set_index(exploded_index, append=True, inplace=True)
@@ -635,51 +691,43 @@ class GeoDataFrame(GeoPandasBase, DataFrame):
         geo_df = df.set_geometry(self._geometry_column_name)
         return geo_df
 
+    # overrides the pandas astype method to ensure the correct return type
+    def astype(self, dtype, copy=True, errors="raise", **kwargs):
+        """
+        Cast a pandas object to a specified dtype ``dtype``.
 
-def points_from_xy(x, y, z=None):
-    """
-    Generate list of shapely Point geometries from x, y(, z) coordinates.
+        Returns a GeoDataFrame when the geometry column is kept as geometries,
+        otherwise returns a pandas DataFrame.
 
-    Parameters
-    ----------
-    x, y, z : array
+        See the pandas.DataFrame.astype docstring for more details.
 
-    Examples
-    --------
-    >>> geometry = geopandas.points_from_xy(x=[1, 0], y=[0, 1])
-    >>> geometry = geopandas.points_from_xy(df['x'], df['y'], df['z'])
-    >>> gdf = geopandas.GeoDataFrame(
-            df, geometry=geopandas.points_from_xy(df['x'], df['y']))
+        Returns
+        -------
+        GeoDataFrame or DataFrame
+        """
+        df = super(GeoDataFrame, self).astype(dtype, copy=copy, errors=errors, **kwargs)
 
-    Returns
-    -------
-    list : list
-    """
-    if not len(x) == len(y):
-        raise ValueError("x and y arrays must be equal length.")
-    if z is not None:
-        if not len(z) == len(x):
-            raise ValueError("z array must be same length as x and y.")
-        geom = [Point(i, j, k) for i, j, k in zip(x, y, z)]
-    else:
-        geom = [Point(i, j) for i, j in zip(x, y)]
-    return geom
+        try:
+            df = geopandas.GeoDataFrame(df, geometry=self._geometry_column_name)
+            return df
+        except TypeError:
+            df = pd.DataFrame(df)
+            return df
 
 
 def _dataframe_set_geometry(self, col, drop=False, inplace=False, crs=None):
     if inplace:
-        raise ValueError("Can't do inplace setting when converting from"
-                         " DataFrame to GeoDataFrame")
+        raise ValueError(
+            "Can't do inplace setting when converting from DataFrame to GeoDataFrame"
+        )
     gf = GeoDataFrame(self)
     # this will copy so that BlockManager gets copied
     return gf.set_geometry(col, drop=drop, inplace=False, crs=crs)
 
+
 if PY3:
     DataFrame.set_geometry = _dataframe_set_geometry
 else:
     import types
-    DataFrame.set_geometry = types.MethodType(_dataframe_set_geometry, None,
-                                              DataFrame)
 
-
-GeoDataFrame._create_indexer('cx', _CoordinateIndexer)
+    DataFrame.set_geometry = types.MethodType(_dataframe_set_geometry, None, DataFrame)
