@@ -1,20 +1,19 @@
-import os
 from distutils.version import LooseVersion
 
-import fiona
 import numpy as np
-
 import six
 
+import fiona
+
+from geopandas import GeoDataFrame, GeoSeries
+
 try:
     from fiona import Env as fiona_env
 except ImportError:
     from fiona import drivers as fiona_env
 
-from geopandas import GeoDataFrame, GeoSeries
 
-
-_FIONA18 = LooseVersion(fiona.__version__) >= LooseVersion('1.8')
+_FIONA18 = LooseVersion(fiona.__version__) >= LooseVersion("1.8")
 
 
 # Adapted from pandas.io.common
@@ -28,14 +27,14 @@ else:
     from urlparse import uses_relative, uses_netloc, uses_params
 
 _VALID_URLS = set(uses_relative + uses_netloc + uses_params)
-_VALID_URLS.discard('')
+_VALID_URLS.discard("")
 
 
 def _is_url(url):
     """Check to see if *url* has a valid protocol."""
     try:
         return parse_url(url).scheme in _VALID_URLS
-    except:
+    except Exception:
         return False
 
 
@@ -79,7 +78,7 @@ def read_file(filename, bbox=None, **kwargs):
             # In a future Fiona release the crs attribute of features will
             # no longer be a dict. The following code will be both forward
             # and backward compatible.
-            if hasattr(features.crs, 'to_dict'):
+            if hasattr(features.crs, "to_dict"):
                 crs = features.crs.to_dict()
             else:
                 crs = features.crs
@@ -98,8 +97,7 @@ def read_file(filename, bbox=None, **kwargs):
     return gdf
 
 
-def to_file(df, filename, driver="ESRI Shapefile", schema=None,
-            **kwargs):
+def to_file(df, filename, driver="ESRI Shapefile", schema=None, **kwargs):
     """
     Write this GeoDataFrame to an OGR data source
 
@@ -121,13 +119,14 @@ def to_file(df, filename, driver="ESRI Shapefile", schema=None,
 
     The *kwargs* are passed to fiona.open and can be used to write
     to multi-layer data, store data within archives (zip files), etc.
+    The path may specify a fiona VSI scheme.
     """
     if schema is None:
         schema = infer_schema(df)
-    filename = os.path.abspath(os.path.expanduser(filename))
     with fiona_env():
-        with fiona.open(filename, 'w', driver=driver, crs=df.crs,
-                        schema=schema, **kwargs) as colxn:
+        with fiona.open(
+            filename, "w", driver=driver, crs=df.crs, schema=schema, **kwargs
+        ) as colxn:
             colxn.writerecords(df.iterfeatures())
 
 
@@ -139,23 +138,28 @@ def infer_schema(df):
 
     def convert_type(column, in_type):
         if in_type == object:
-            return 'str'
-        if in_type.name.startswith('datetime64'):
+            return "str"
+        if in_type.name.startswith("datetime64"):
             # numpy datetime type regardless of frequency
-            return 'datetime'
+            return "datetime"
         out_type = type(np.zeros(1, in_type).item()).__name__
-        if out_type == 'long':
-            out_type = 'int'
-        if not _FIONA18 and out_type == 'bool':
-            raise ValueError('column "{}" is boolean type, '.format(column) +
-                             'which is unsupported in file writing with fiona '
-                             '< 1.8. Consider casting the column to int type.')
+        if out_type == "long":
+            out_type = "int"
+        if not _FIONA18 and out_type == "bool":
+            raise ValueError(
+                'column "{}" is boolean type, '.format(column)
+                + "which is unsupported in file writing with fiona "
+                "< 1.8. Consider casting the column to int type."
+            )
         return out_type
 
-    properties = OrderedDict([
-        (col, convert_type(col, _type)) for col, _type in
-        zip(df.columns, df.dtypes) if col != df._geometry_column_name
-    ])
+    properties = OrderedDict(
+        [
+            (col, convert_type(col, _type))
+            for col, _type in zip(df.columns, df.dtypes)
+            if col != df._geometry_column_name
+        ]
+    )
 
     if df.empty:
         raise ValueError("Cannot write empty DataFrame to file.")
@@ -164,7 +168,7 @@ def infer_schema(df):
     # Fiona allows a list of geometry types
     geom_types = _geometry_types(df)
 
-    schema = {'geometry': geom_types, 'properties': properties}
+    schema = {"geometry": geom_types, "properties": properties}
 
     return schema
 
@@ -182,8 +186,7 @@ def _geometry_types(df):
         geom_types_2D = df[~df.geometry.has_z].geometry.geom_type.unique()
         geom_types_2D = [gtype for gtype in geom_types_2D if gtype is not None]
         geom_types_3D = df[df.geometry.has_z].geometry.geom_type.unique()
-        geom_types_3D = ["3D " + gtype for gtype in geom_types_3D
-                         if gtype is not None]
+        geom_types_3D = ["3D " + gtype for gtype in geom_types_3D if gtype is not None]
         geom_types = geom_types_3D + geom_types_2D
 
     else:
@@ -196,7 +199,7 @@ def _geometry_types(df):
     if len(geom_types) == 0:
         # Default geometry type supported by Fiona
         # (Since https://github.com/Toblerity/Fiona/issues/446 resolution)
-        return 'Unknown'
+        return "Unknown"
 
     if len(geom_types) == 1:
         geom_types = geom_types[0]
@@ -209,13 +212,14 @@ def _geometry_types_back_compat(df):
     for backward compatibility with Fiona<1.8 only
     """
     unique_geom_types = df.geometry.geom_type.unique()
-    unique_geom_types = [
-        gtype for gtype in unique_geom_types if gtype is not None]
+    unique_geom_types = [gtype for gtype in unique_geom_types if gtype is not None]
 
     # merge single and Multi types (eg Polygon and MultiPolygon)
     unique_geom_types = [
-        gtype for gtype in unique_geom_types
-        if not gtype.startswith('Multi') or gtype[5:] not in unique_geom_types]
+        gtype
+        for gtype in unique_geom_types
+        if not gtype.startswith("Multi") or gtype[5:] not in unique_geom_types
+    ]
 
     if df.geometry.has_z.any():
         # declare all geometries as 3D geometries
